diff --git a/xen/common/sched_credit.c b/xen/common/sched_credit.c
index 3ea0d40afb..29076e362b 100644
--- a/xen/common/sched_credit.c
+++ b/xen/common/sched_credit.c
@@ -2005,9 +2005,9 @@ out:
      */
     ret.time = (is_idle_vcpu(snext->vcpu) ?
                 -1 : tslice);
-    ret.task = snext->vcpu;
+    ret.task = snext->vcpu->sched_item;
 
-    CSCHED_VCPU_CHECK(ret.task);
+    CSCHED_VCPU_CHECK(ret.task->vcpu);
     return ret;
 }
 
diff --git a/xen/common/sched_credit2.c b/xen/common/sched_credit2.c
index 8bee7cb9a2..9bf045d20f 100644
--- a/xen/common/sched_credit2.c
+++ b/xen/common/sched_credit2.c
@@ -3636,9 +3636,9 @@ csched2_schedule(
      * Return task to run next...
      */
     ret.time = csched2_runtime(ops, cpu, snext, now);
-    ret.task = snext->vcpu;
+    ret.task = snext->vcpu->sched_item;
 
-    CSCHED2_VCPU_CHECK(ret.task);
+    CSCHED2_VCPU_CHECK(ret.task->vcpu);
     return ret;
 }
 
diff --git a/xen/common/sched_null.c b/xen/common/sched_null.c
index 5570cc1a8c..62c51e2c83 100644
--- a/xen/common/sched_null.c
+++ b/xen/common/sched_null.c
@@ -746,10 +746,10 @@ static struct task_slice null_schedule(const struct scheduler *ops,
     if ( tasklet_work_scheduled )
     {
         trace_var(TRC_SNULL_TASKLET, 1, 0, NULL);
-        ret.task = idle_vcpu[cpu];
+        ret.task = idle_vcpu[cpu]->sched_item;
     }
     else
-        ret.task = per_cpu(npc, cpu).vcpu;
+        ret.task = per_cpu(npc, cpu).vcpu->sched_item;
     ret.migrated = 0;
     ret.time = -1;
 
@@ -784,7 +784,7 @@ static struct task_slice null_schedule(const struct scheduler *ops,
                 {
                     vcpu_assign(prv, wvc->vcpu, cpu);
                     list_del_init(&wvc->waitq_elem);
-                    ret.task = wvc->vcpu;
+                    ret.task = wvc->vcpu->sched_item;
                     goto unlock;
                 }
             }
@@ -793,10 +793,10 @@ static struct task_slice null_schedule(const struct scheduler *ops,
         spin_unlock(&prv->waitq_lock);
     }
 
-    if ( unlikely(ret.task == NULL || !vcpu_runnable(ret.task)) )
-        ret.task = idle_vcpu[cpu];
+    if ( unlikely(ret.task == NULL || !item_runnable(ret.task)) )
+        ret.task = idle_vcpu[cpu]->sched_item;
 
-    NULL_VCPU_CHECK(ret.task);
+    NULL_VCPU_CHECK(ret.task->vcpu);
     return ret;
 }
 
diff --git a/xen/common/sched_rt.c b/xen/common/sched_rt.c
index 0639cdce0a..374a9d2383 100644
--- a/xen/common/sched_rt.c
+++ b/xen/common/sched_rt.c
@@ -1138,7 +1138,7 @@ rt_schedule(const struct scheduler *ops, s_time_t now, bool_t tasklet_work_sched
         }
         ret.time = snext->cur_budget; /* invoke the scheduler next time */
     }
-    ret.task = snext->vcpu;
+    ret.task = snext->vcpu->sched_item;
 
     return ret;
 }
diff --git a/xen/common/schedule.c b/xen/common/schedule.c
index ffc0920e5f..2a2f36636a 100644
--- a/xen/common/schedule.c
+++ b/xen/common/schedule.c
@@ -1534,7 +1534,7 @@ static void schedule(void)
     sched = this_cpu(scheduler);
     next_slice = sched->do_schedule(sched, now, tasklet_work_scheduled);
 
-    next = next_slice.task;
+    next = next_slice.task->vcpu;
 
     sd->curr = next->sched_item;
 
diff --git a/xen/include/xen/sched-if.h b/xen/include/xen/sched-if.h
index 51c3477580..295101f9ef 100644
--- a/xen/include/xen/sched-if.h
+++ b/xen/include/xen/sched-if.h
@@ -176,9 +176,9 @@ static inline spinlock_t *pcpu_schedule_trylock(unsigned int cpu)
 }
 
 struct task_slice {
-    struct vcpu *task;
-    s_time_t     time;
-    bool_t       migrated;
+    struct sched_item *task;
+    s_time_t           time;
+    bool_t             migrated;
 };
 
 struct scheduler {

diff --git a/xen/common/sched_arinc653.c b/xen/common/sched_arinc653.c
index 9dc1ff6a73..5733a2a6b8 100644
--- a/xen/common/sched_arinc653.c
+++ b/xen/common/sched_arinc653.c
@@ -554,9 +554,9 @@ a653sched_do_schedule(
 
     /*
      * If there are more domains to run in the current major frame, set
      * new_task equal to the address of next domain's VCPU structure.
      * Otherwise, set new_task equal to the address of the idle task's VCPU
      * structure.
      */
     new_task = (sched_index < sched_priv->num_schedule_entries)
         ? sched_priv->schedule[sched_index].vc
@@ -592,7 +592,7 @@ a653sched_do_schedule(
      * of the selected task's VCPU structure.
      */
     ret.time = next_switch_time - now;
-    ret.task = new_task;
+    ret.task = new_task->sched_item;
     ret.migrated = 0;
 
     BUG_ON(ret.time <= 0);
@@ -554,9 +554,9 @@ a653sched_do_schedule(
     /*
      * If there are more domains to run in the current major frame, set
-     * new_task equal to the address of next domain's VCPU structure.
-     * Otherwise, set new_task equal to the address of the idle task's VCPU
-     * structure.
+     * new_task equal to the address of next domain's sched_item structure.
+     * Otherwise, set new_task equal to the address of the idle task's
+     * sched_item structure.
      */
     new_task = (sched_index < sched_priv->num_schedule_entries)